@@ -97,7 +97,7 @@ sub begin {
     $file = File::Spec->rel2abs( $file );
 
     # create the directory
-    my $dir = File::Spec->catdir( (File::Spec->splitpath($file))[ 0, 1 ] );
+    my $dir = File::Spec->catpath( (File::Spec->splitpath($file))[ 0, 1 ], '' );
     if( ! -e $dir ) {
         eval { mkpath( $dir ) };
         if ($@) {
@@ -184,13 +184,13 @@ HTTP::Proxy::BodyFilter::save - A filter that saves transfered data to a file
 
     # save RFC files as we browse them
     $proxy->push_filter(
-        path => qr!/rfc\d+.txt!,
-        mime => 'text/plain',
+        path     => qr!/rfc\d+.txt!,
+        mime     => 'text/plain',
         response => HTTP::Proxy::BodyFilter::save->new(
             template => '%f',
             prefix   => 'rfc',
             keep_old => 1,
-        );
+        )
     );
 
     $proxy->start;
